﻿using Kivii.DataAnnotations;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Kivii.Finances.Entities
{
    [Api(Description = "账户")]
    [Alias(Configs.TableNameAccount)]
    public class Account : EntityWithMetadata, IEntityInAssemblyDb,
        IEntityHasCreator
    {
        #region 账户所有人，禁止更新
        [IgnoreUpdate]
        [DefaultEmptyGuid]
        public Guid BizKvid { get; set; }

        [ApiMember(Description = "业务流水号")]
        [IgnoreUpdate]
        [StringLength(200)]
        public string BizId { get; set; }

        [ApiMember(Description = "业务类型")]
        [IgnoreUpdate]
        [StringLength(100)]
        public string BizType { get; set; }

        [IgnoreUpdate]
        [DefaultEmptyGuid, Required]
        public Guid OwnerKvid { get; set; }

        [ApiMember(Description = "所有者名称")]
        [IgnoreUpdate]
        [StringLength(500), Required]
        public string OwnerName { get; set; }
        #endregion

        #region 帐户相关 账户类型禁止更新

        [ApiMember(Description = "账户名称,如果是Bank类型，可以写银行名称，如果是其它类型，这个应该是账户的友好名称，不应该和OwnerName重复")]
        [StringLength(100)]
        [Required]
        public string Name { get; set; }

        [ApiMember(Description = "账户编号")]
        [StringLength(100),Default("")]
        public string SerialNumber { get; set; }


        [ApiMember(Description = "类型，账户类型")]
        [StringLength(50), Required]
        public AccountType Type { get; set; }

        [ApiMember(Description = "失效日期，仅针对StoredCard储值卡 和 Coupon 优惠券")]
        public DateTime? ExpiredDate { get; set; }
        #endregion

        #region 金额相关，账户单位禁止更新，额度和金额都是内部更新

        [ApiMember(Description = "授信额度，仅针对存款账户")]
        [InternalSetter]
        [DecimalLength(15, 2), Default(0)]
        public decimal CreditLine { get; set; }

        [ApiMember(Description = "当前金额")]
        [InternalSetter]
        [DecimalLength(15, 2), Required]
        public decimal Amount { get; set; }

        [ApiMember(Description = "单位")]
        [IgnoreUpdate]
        [StringLength(50), Required]
        public CurrencyUnit Currency { get; set; }

        #endregion

        #region 文本 摘要 备注
        [ApiMember(Description = "摘要，最大500字")]
        [StringLength(500), Default("")]
        public string Summary { get; set; }

        [ApiMember(Description = "备注，最大2000字")]
        [StringLength(2000), Default("")]
        public string Remark { get; set; }
        #endregion

        #region 操作相关  Creator

        [ApiMember(Description = "创建人Kvid ")]
        [IgnoreUpdate]
        [CurrentMemberKvid]
        public Guid CreatorKvid { get; set; }

        [ApiMember(Description = "创建人")]
        [IgnoreUpdate]
        [StringLength(50), CurrentMemberName]
        public string CreatorName { get; set; }

        [ApiMember(Description = "组织机构名称,当组织有子组织时区分所属组织")]
        [IgnoreUpdate]
        [StringLength(200),CurrentOrganizationName]
        public string OrganizationName { get; set; }

        [ApiMember(Description = "组织机构Id,当组织有子组织时区分所属组织")]
        [IgnoreUpdate]
        [CurrentOrganizationKvid]
        public Guid OrganizationKvid { get; set; }
        #endregion
    }

    /// <summary>
    /// 账户类型
    /// </summary>
    public enum AccountType
    {
        /// <summary>
        /// 不支持的账户类型
        /// </summary>
        Unsupported = 0,

        ///// <summary>
        /////银行账户，可清零或调帐，不用于客户，只用于组织内
        ///// </summary>
        //Bank,

        //AliPay,

        //WeChat,

        /// <summary>
        ///存款账户，指客户的存款账户,同部门只能拥有一个, 多部门可存在多个账户
        /// </summary>
        Deposit,

        /// <summary>
        ///优惠券,一次性使用，多不退少补
        /// </summary>
        Coupon,

        /// <summary>
        ///储值卡，只能充值和消费，不能转出
        /// </summary>
        StoredCard,

        /// <summary>
        ///业务专用账户，可设多个业务账户，用于记录不同业务的收入
        /// </summary>
        Biz,

        /// <summary>
        /// 平衡系统所有账户金额为0,一个系统仅有一个,即系统与系统外部有金额往来就新增账户明细
        /// </summary>
        Balance,

        /// <summary>
        ///现金账户，用于收到的现金，只用于组织内，个人业务员使用,不用于客户，如业务员收取现金
        /// </summary>
        Cash,

        /// <summary>
        ///刷卡账户,用于收到的刷卡,只用于组织内,个人业务员使用
        /// </summary>
        Pos,

        /// <summary>
        /// 折扣账户,各部门仅有一个,也可所属到个人
        /// </summary>
        Discount,

        /// <summary>
        /// 退款账户,所属于系统使用退款账户
        /// </summary>
        Refund
    }
}
