﻿using Kivii.Finances.Entities;
using Kivii.Linq;
using Kivii.Web;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Kivii.Finances.Transforms
{
    public enum OwnerType
    {
        Unsupport=0,
        Department = 1,
        Organization = 2,
        Member = 3
    }

    [Api(Description = "业务认领")]
    [RequiresAnyRole(SystemRoles.Everyone)]
    public class PaymentSplit : RestfulExecution<Payment>
    {
        [ApiMember(Description = "认领金额")]
        public decimal Amount { get; set; }

        [ApiMember(Description = "认领类型,默认为Department,Organization,Member")]
        public OwnerType OwnerType { get; set; } = OwnerType.Department;

        public Guid Kvid { get; set; }

        public override object OnExecution(IRequest req, IResponse res)
        {
            if (Kvid == Guid.Empty) throw new Exception("请选择要认领的收款!");

            var conn = KiviiContext.GetOpenedDbConnection<Payment>();

            var payment = conn.SingleById<Payment>(Kvid);
            if (payment == null) throw new Exception("收款不存在!");
            if (payment.Type != PaymentType.Bank && payment.Type != PaymentType.Split) throw new Exception("非银行到账不可以认领!");
            if (payment.OffsetKvid != Guid.Empty) throw new Exception("已冲销收款不可认领!");

            var ownerKvid = Guid.Empty;
            var ownerName = string.Empty;
            switch (OwnerType)
            {
                case OwnerType.Organization:
                    {
                        ownerKvid = KiviiContext.CurrentMember.OrganizationKvid;
                        ownerName = KiviiContext.CurrentMember.OrganizationName;
                        break;
                    }
                case OwnerType.Member:
                    {
                        ownerKvid = KiviiContext.CurrentMember.Kvid;
                        ownerName = KiviiContext.CurrentMember.FullName;
                        break;
                    }
                default:
                    {
                        ownerKvid = KiviiContext.CurrentMember.DepartmentKvid;
                        ownerName = KiviiContext.CurrentMember.DepartmentName;
                        break;
                    }
            }

            Payment bank = null;
            Payment split = null;
            EntityLog<Payment> log = null;

            if (payment.Type == PaymentType.Bank)
            {
                if (Amount <= 0) throw new Exception("金额不能为小于0!");
                if (payment.AmountSplited == payment.Amount) throw new Exception("无剩余收款可认领!");
                if (payment.AmountSplited + Amount > payment.Amount) throw new Exception("认领金额超出范围!");
                var amountSplit = conn.Scalar<Payment, decimal>(o => Sql.Sum(o.Amount), p => p.ParentKvid == payment.Kvid && p.OffsetKvid == Guid.Empty);
                if (amountSplit + Amount > payment.Amount) throw new Exception("认领金额超出范围!");

                bank = payment;
                split = bank.BizSplit(Amount, ownerKvid, ownerName, conn);

                #region 记录日志
                log = new EntityLog<Payment>();
                log.OwnerKvid = bank.Kvid;
                log.BizId = split.SerialNumber;
                log.BizKvid = split.Kvid;
                log.BizType = typeof(Payment).FullName;
                log.Title = DateTime.Now.ToString("yyyy-MM-dd HH:mm:ss");
                log.Remark = KiviiContext.CurrentMember.FullName + $"在  {log.Title} 对流水号为:{bank.SerialNumber}的收款进行认领至{ownerName},认领金额为:{split.Amount}元.";
                #endregion
            }

            if (payment.Type == PaymentType.Split)
            {
                split = payment;

                split.OwnerKvid = ownerKvid;
                split.AddOnlyProperties(o => o.OwnerKvid);
                split.OwnerName = ownerName;
                split.AddOnlyProperties(o => o.OwnerName);

                #region 记录日志
                log = new EntityLog<Payment>();
                log.OwnerKvid = split.ParentKvid;
                log.BizId = split.SerialNumber;
                log.BizKvid = split.Kvid;
                log.BizType = typeof(Payment).FullName;
                log.Title = DateTime.Now.ToString("yyyy-MM-dd HH:mm:ss");
                log.Remark = KiviiContext.CurrentMember.FullName + $"在  {log.Title} 对流水号为:{split.SerialNumber}的收款进行认领至{ownerName},认领金额为:{split.Amount}元.";
                #endregion
            }
            var rtns = new RestfulExecutionResponse<Payment>();
            rtns.Results = new List<Payment>();

            //开启事务进行数据库操作
            var trans = conn.OpenTransaction();
            try
            {
                if (payment.Type == PaymentType.Bank)
                {
                    if (split != null)
                    {
                        conn.Insert(split);
                        rtns.Results.Add(split);
                    }
                    payment.RecalculateAmountSplit(conn);

                    if (log != null) conn.Insert(log);
                }
                if (payment.Type == PaymentType.Split)
                {
                    if (split != null)
                    {
                        conn.UpdateOnly(split);
                        rtns.Results.Add(split);
                    }
                    if (log != null) conn.Insert(log);
                }

                trans.Commit();
            }
            catch (Exception ex)
            {
                trans.Rollback();
                throw ex;
            }
            return rtns;
        }
    }

    [Api(Description = "批量业务认领")]
    [RequiresAnyRole(SystemRoles.Everyone)]
    public class PaymentMultiSplit : RestfulExecution<Payment>
    {
        [ApiMember(Description = "认领类型,默认为Department,Organization,Member")]
        public OwnerType OwnerType { get; set; } = OwnerType.Department;

        public List<Guid> Kvids { get; set; }
        public override object OnExecution(IRequest req, IResponse res)
        {
            if (Kvids.IsNullOrEmpty()) throw new Exception("请选择要认领的收款!");

            var conn = KiviiContext.GetOpenedDbConnection<Payment>();

            var payments = conn.SelectByIds<Payment>(Kvids);
            if (payments == null) throw new Exception("收款不存在!");
            if (payments.Exists(o => o.Type != PaymentType.Bank && o.Type != PaymentType.Split)) throw new Exception("非银行到账不可以认领!");
            if (payments.Exists(o => o.OffsetKvid != Guid.Empty)) throw new Exception("已冲销收款不可认领!");
            if (payments.Exists(o => o.AmountSplited >= o.Amount)) throw new Exception("无剩余收款可认领!");

            var ownerKvid = Guid.Empty;
            var ownerName = string.Empty;
            switch (OwnerType)
            {
                case OwnerType.Department:
                    {
                        ownerKvid = KiviiContext.CurrentMember.DepartmentKvid;
                        ownerName = KiviiContext.CurrentMember.DepartmentName;
                        break;
                    }
                case OwnerType.Organization:
                    {
                        ownerKvid = KiviiContext.CurrentMember.OrganizationKvid;
                        ownerName = KiviiContext.CurrentMember.OrganizationName;
                        break;
                    }
                case OwnerType.Member:
                    {
                        ownerKvid = KiviiContext.CurrentMember.Kvid;
                        ownerName = KiviiContext.CurrentMember.FullName;
                        break;
                    }
            }

            var banks = new List<Payment>();
            var splits = new List<Payment>();
            var splitsSetOwner = new List<Payment>();
            var logs = new List<EntityLog<Payment>>();

            foreach (var payment in payments)
            {
                if (payment.Type == PaymentType.Bank)
                {
                    var amount = payment.Amount - payment.AmountSplited;
                    if (amount <= 0) continue;
                    var splitPayment = payment.BizSplit(amount, ownerKvid, ownerName, conn);
                    splits.Add(splitPayment);

                    banks.Add(payment);

                    #region 记录日志
                    var log = new EntityLog<Payment>();
                    log.OwnerKvid = payment.Kvid;
                    log.BizId = splitPayment.SerialNumber;
                    log.BizKvid = splitPayment.Kvid;
                    log.BizType = typeof(Payment).FullName;
                    log.Title = DateTime.Now.ToString("yyyy-MM-dd HH:mm:ss");
                    log.Remark = KiviiContext.CurrentMember.FullName + $"在  {log.Title} 对流水号为:{payment.SerialNumber}的收款进行认领至{ownerName},认领金额为:{splitPayment.Amount}元.";
                    logs.Add(log);
                    #endregion
                }
                if (payment.Type == PaymentType.Split)
                {
                    payment.OwnerKvid = ownerKvid;
                    payment.AddOnlyProperties(o => o.OwnerKvid);
                    payment.OwnerName = ownerName;
                    payment.AddOnlyProperties(o => o.OwnerName);
                    splitsSetOwner.Add(payment);

                    #region 记录日志
                    var log = new EntityLog<Payment>();
                    log.OwnerKvid = payment.ParentKvid;
                    log.BizId = payment.SerialNumber;
                    log.BizKvid = payment.Kvid;
                    log.BizType = typeof(Payment).FullName;
                    log.Title = DateTime.Now.ToString("yyyy-MM-dd HH:mm:ss");
                    log.Remark = KiviiContext.CurrentMember.FullName + $"在  {log.Title} 对流水号为:{payment.SerialNumber}的收款进行认领至{ownerName},认领金额为:{payment.Amount}元.";
                    logs.Add(log);
                    #endregion
                }
            }

            var rtns = new RestfulExecutionResponse<Payment>();
            rtns.Results = new List<Payment>();

            //开启事务进行数据库操作
            var trans = conn.OpenTransaction();
            try
            {
                foreach (var split in splits)
                {
                    conn.Insert(split);
                    rtns.Results.Add(split);
                }
                foreach (var item in splitsSetOwner)
                {
                    conn.UpdateOnly(item);
                    rtns.Results.Add(item);
                }
                foreach (var item in banks)
                {
                    item.RecalculateAmountSplit(conn);
                }
                foreach (var log in logs)
                {
                    conn.Insert(log);
                }

                trans.Commit();
            }
            catch (Exception ex)
            {
                trans.Rollback();
                throw ex;
            }
            return rtns;
        }
    }

    [Api(Description = "认领撤销")]
    [RequiresAnyRole(SystemRoles.Everyone)]
    public class PaymentSplitCancel : RestfulExecution<Payment>
    {
        public List<Guid> Kvids { get; set; }

        public string Remark { get; set; }

        public override object OnExecution(IRequest req, IResponse res)
        {
            if (Kvids.IsNullOrEmpty()) throw new Exception("请选择要撤销认领的收款!");

            var conn = KiviiContext.GetOpenedDbConnection<Payment>();
            var payments = conn.SelectByIds<Payment>(Kvids);
            if (payments == null) throw new Exception("收款不存在!");
            if (payments.Exists(o => o.Type != PaymentType.Split)) throw new Exception("非银行到账不可以认领!");
            if (payments.Exists(o => o.OffsetKvid != Guid.Empty)) throw new Exception("已冲销收款不可撤销认领!");
            (conn.Exists<InvoiceApply>(o => o.OperateType == InvoiceApplyType.Related && o.OffsetKvid == Guid.Empty && o.AmountInvoice != o.Amount && Sql.In(o.BizKvid, payments.ConvertAll(p => p.Kvid)))).ThrowIfTrue("存在申请开票中的到账,暂无法撤销认领!");
            var rtns = new RestfulExecutionResponse<Payment>();
            rtns.Results = new List<Payment>();
            //if (payments.Exists(o => o.AmountUsed > 0)) throw new Exception("存在到账已使用,不可撤销认领!");
            //认领撤销分两种情况.
            //第一种:要是要撤销的目标已经关联过业务了被使用了,撤销只会清空所属信息,重新认领则只是赋值所属信息
            //第二种:要是没有被业务关联没被使用,撤销则是Offset当前拆分项目,重新认领则会生成新的拆分项

            var paymentUnuseds = payments.Where(o => o.AmountUsed == 0).ToList();
            var paymentUseds = payments.Where(o => o.AmountUsed > 0).ToList();

            #region 已经使用了的撤销
            if (!paymentUseds.IsNullOrEmpty())
            {
                var paymentKvids = paymentUseds.ConvertAll(o => o.Kvid);

                var updateLamda = conn.From<Payment>();
                updateLamda = updateLamda.Update(o => new { o.OwnerKvid, o.OwnerName });
                updateLamda = updateLamda.Where(o => Sql.In(o.Kvid, paymentKvids));
                conn.UpdateOnly(new Payment { OwnerKvid = Guid.Empty, OwnerName = string.Empty }, updateLamda);
                rtns.Results.AddRange(paymentUseds);
            }
            #endregion

            #region 没有使用的撤销
            if (!paymentUnuseds.IsNullOrEmpty())
            {
                var parentKvids = paymentUnuseds.ConvertAll(o => o.ParentKvid).Distinct().ToList();
                var parents = conn.SelectByIds<Payment>(parentKvids);

                var parentUpdates = new List<Payment>();
                var splitOffsets = new List<Payment>();
                var offsets = new List<Payment>();

                foreach (var split in paymentUnuseds)
                {
                    var parent = parents.FirstOrDefault(o => o.Kvid == split.ParentKvid);
                    if (parent == null) continue;
                    
                    var offset = split.Offset($"认领撤销:{Remark}");
                    offsets.Add(offset);
                    splitOffsets.Add(split);
                    parentUpdates.Add(parent);
                }

                var trans = conn.OpenTransaction();
                try
                {
                    foreach (var item in splitOffsets)
                    {
                        conn.UpdateOnly(item);
                        rtns.Results.Add(item);
                    }
                    foreach (var item in offsets)
                    {
                        conn.Insert(item);
                    }
                    foreach (var item in parentUpdates)
                    {
                        item.RecalculateAmountSplit(conn);
                    }
                    trans.Commit();
                }
                catch (Exception ex)
                {
                    trans.Rollback();
                    throw ex;
                }
            }

            #endregion

            return rtns;
        }
    }
}
